"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Build Forge 
(c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: WasDeploy.py
	
	Encapsulates useful utilities used for WebSphere specific application deployment
"""

from Logger import _Logger
from SystemUtils import SystemUtils
from JavaConversionHelper import JavaConversionHelper

import AdminConfig
import AdminApp



"""
This options filter is used by default in the WasDeploy class to 
filter out options.  Implementors can supply a custom options filter
to filter options.
"""
class NoOpOptionsFilter:
	
	"""
		Includes all options, always returns true
		
		name is the full property name
		value is the property value
		wasOption is the WAS specific name of the option
		
		Return 1(true) to include the option
		Return 0(false) to exclude the option
	"""
	def includeOption(self, name, value, wasOption):
		return 1
	#endDef
	
#endClass

"""
CLASS :: _WasDeploy
Encapsulates useful utilities used for                 
WebSphere specific application deployment                
"""
class _WasDeploy:
	
	def __init__(self):
		self.LOGGER = _Logger("WasDeploy")
	#endDef
	
	"""
		Based on the contents of the propsFile, creates an options string
		which is used to deploy the application to WAS
	"""
	def createOptions(self, propsFile):
		props = SystemUtils.getPropertyDict(propsFile)
		return self.createOptionsFromDict(props, "")
	#endDef
	
	"""
		Based on the contents of the supplied props dictionary,
		creates an options string which is used to deploy or update
		the application in WAS
	"""
	def createOptionsFromDict(self, props, keyPrefix="", optionsFilter = NoOpOptionsFilter()):
		optionsDict = {}
		optionsStr = ""
		for prop in props.keys():
			if (prop.find(keyPrefix + "options.multi.") > -1):
				# key starts with options.multi
				# process as array
				# keys will be index.name, so we parse off the index portion
				# options.multi. is 14 chars in length
				prefixLength = 14 + len(keyPrefix)
				indexKey = prop[prefixLength:len(prop)]
				value = props[prop].strip()
				wasOption = indexKey.split('.',1)[1]
				if (optionsFilter.includeOption(prop, value, wasOption)):
					key = "-" + wasOption
					if (not optionsDict.has_key(key)):
						optionsDict[key] = []
					#endIf
					optionsDict[key].extend(str(value))
				else:
					self.LOGGER.info("optionsFilter filtered and skipped key: " + prop + " with value: " + value)
				#endIf
			elif (prop.find(keyPrefix + "options.") > -1):
				# key starts with options
				# process as single value
				# options. is 8 chars in length
				prefixLength = 8 + len(keyPrefix)
				key = prop[prefixLength:len(prop)]
				value = props[prop].strip()
				if (optionsFilter.includeOption(prop, value, key)):
					if (self._isBoolean(value)):
						if (value.lower() == "true" or value.lower() == "yes"):
							optionsStr += "-" + key + " "
						else:
							optionsStr += "-no" + key + " "
						#endIf
					else:
						optionsStr += "-" + key + " " + value + " "
					#endIf
				else:
					self.LOGGER.info("optionsFilter filtered and skipped key: " + prop + " with value: " + value)
				#endIf
			#endIf
		#endFor
		## optionsDict contains []s for values, append those to the options string
		for key in optionsDict.keys():
			optionsStr += key + " ["
			for value in optionsDict[key]:
				optionsStr += value
			optionsStr += "] "
		#endFor	
		return optionsStr
	#endDef
	
	"""
		Returns true if the supplied string represents a boolean.
		
		A string represents a boolean if its value is true|false|yes|no ignoring case
	"""
	def _isBoolean(self, value):
		lowerVal = value.lower()
		if (lowerVal == "true" or lowerVal == "false" or lowerVal == "yes" or lowerVal == "no"):
			return 1
		else:
			return 0
	#endDef
	
	"""
	checkIfAppExists 
		Description:                                     
			Return true if the application exists (has been deployed)
			to this WebSphere environment
	                                                        
		Inputs:                                              
			appName = the name of the application in WebSphere
	"""
	def checkIfAppExists (self, appName ):
		appExists = 1
		application = AdminConfig.getid("/Deployment:"+appName+"/" ).strip()
		if (len(application) == 0):
			print "Info: Application with name " + str(appName) + " does not exist in Application Server configuration" 
			appExists = 0
		#endIf 
		return appExists
	#endDef 
	
	def postInstallConfig ( self, appName, warClassLoaderPolicy, classLoaderMode ):
		application = AdminConfig.getid("/Deployment:"+appName+"/" ).strip()
		if (len(application) > 0):
			deployedObject = AdminConfig.showAttribute(application, "deployedObject" )
			classloader = AdminConfig.showAttribute(deployedObject, 'classloader')

			if (warClassLoaderPolicy is None or warClassLoaderPolicy == "SINGLE"):
				AdminConfig.modify(deployedObject, [["warClassLoaderPolicy", "SINGLE"]] )
			else:
				AdminConfig.modify(deployedObject, [["warClassLoaderPolicy", "MULTIPLE"]] )
			#endElse 
			if (classLoaderMode is not None and classLoaderMode == "PARENT_LAST"):
				AdminConfig.modify(classloader, [["mode", "PARENT_LAST"]] )
			else:
				AdminConfig.modify(classloader, [["mode", "PARENT_FIRST"]] )
			#endIf
		else:
			msg = "Application with name " + str(appName) + " does not exist in Application Server configuration" 
			self.LOGGER.error(msg)
			raise Exception(msg)
		#endIf
	#endDef 
	
	def installApp ( self, options, scopeArguments, appFile, appName):
		#available installable apps
		appExists = self.checkIfAppExists(appName )
		if (appExists):
			# update the app
			print "Updating "+appName+" ..."
			optionsAsSequence = JavaConversionHelper.convertStringToSequence('-update -appname '+appName+' -verbose ' + scopeArguments + ' '+options)
			AdminApp.install(appFile, optionsAsSequence)
		else:
			# install the apps
			print "Installing "+appName+" ..."
			optionsAsSequence = JavaConversionHelper.convertStringToSequence('-appname '+appName+' -verbose ' + scopeArguments + ' '+options)
			AdminApp.install(appFile, optionsAsSequence)
		#endElse 
	#endDef 
	
#endClass


## define global singletons
WasDeploy = _WasDeploy()	

